"use strict";
var _a = require('eosjs'), Api = _a.Api, JsonRpc = _a.JsonRpc;
var JsSignatureProvider = require('eosjs/dist/eosjs-jssig').JsSignatureProvider;
var fetch = require('node-fetch');
var _b = require('text-encoding'), TextDecoder = _b.TextDecoder, TextEncoder = _b.TextEncoder;
var accounts = require('./accounts');
var cpu = require('./tokens/cpu');
var compose = require('./compose');
var createbridge = require('./createbridge');
var crypto = require('./modules/crypto');
var eos = require('./eos');
var errors = require('./errors');
var helpers = require('./helpers');
var instrument = require('./instrument');
var ore = require('./tokens/ore');
var oreStandardToken = require('./orestandardtoken');
var rightsRegistry = require('./rightsregistry');
var usageLog = require('./usagelog');
var verifier = require('./verifier');
var Orejs = /** @class */ (function () {
    function Orejs(config) {
        if (config === void 0) { config = {}; }
        this.constructEos(config);
        /* Mixins */
        Object.assign(this, accounts);
        Object.assign(this, compose);
        Object.assign(this, cpu);
        Object.assign(this, createbridge);
        Object.assign(this, crypto);
        Object.assign(this, eos);
        Object.assign(this, errors);
        Object.assign(this, helpers);
        Object.assign(this, instrument);
        Object.assign(this, ore);
        Object.assign(this, oreStandardToken);
        Object.assign(this, rightsRegistry);
        Object.assign(this, usageLog);
        Object.assign(this, verifier);
    }
    Orejs.prototype.constructEos = function (config) {
        this.config = config;
        this.chainName = config.chainName || 'ore'; // ore || eos
        this.unusedAccountPubKey = config.unusedAccountPubKey || null;
        this.rpc = new JsonRpc(config.httpEndpoint, { fetch: config.fetch || fetch });
        this.signatureProvider = config.signatureProvider || new JsSignatureProvider(config.privateKeys || []);
        this.eos = new Api({
            chainId: config.chainId,
            rpc: this.rpc,
            signatureProvider: this.signatureProvider,
            textEncoder: new TextEncoder(),
            textDecoder: new TextDecoder()
        });
    };
    return Orejs;
}());
var generateAccountNameString = accounts.generateAccountNameString;
module.exports = {
    compose: compose,
    crypto: crypto,
    errors: errors,
    generateAccountNameString: generateAccountNameString,
    hexToUint8Array: eos.hexToUint8Array,
    isValidPublicKey: eos.isValidPublicKey,
    Orejs: Orejs,
    JsSignatureProvider: JsSignatureProvider
};
//# sourceMappingURL=index.js.map